@@ -108,7 +108,7 @@ addons_install_response (GtkWidget*  infobar,
         if (uri && *uri)
         {
             gchar** split_uri;
-            gchar* path, *filename, *hostname, *dest_path, *temp_uri;
+            gchar* path, *filename, *hostname, *dest_path, *temp_uri, *folder_path;
             const gchar* folder;
             WebKitNetworkRequest* request;
             WebKitDownload* download;
@@ -119,7 +119,11 @@ addons_install_response (GtkWidget*  infobar,
             filename = NULL;
             folder = NULL;
 
-            if (!g_strcmp0 (hostname, "userscripts.org"))
+            if (g_str_has_suffix (uri, ".user.js"))
+                folder = "scripts";
+            else if (g_str_has_suffix (uri, ".user.css"))
+                folder = "styles";
+            else if (!g_strcmp0 (hostname, "userscripts.org"))
             {
                 gchar* script_id;
                 const gchar* js_script;
@@ -129,7 +133,7 @@ addons_install_response (GtkWidget*  infobar,
                 web_view = WEBKIT_WEB_VIEW (midori_view_get_web_view (view));
                 web_frame = webkit_web_view_get_main_frame (web_view);
 
-                js_script = "document.getElementById('heading').childNodes[3].childNodes[1].innerHTML";
+                js_script = "document.getElementById('heading').childNodes[3].childNodes[1].textContent";
                 if (WEBKIT_IS_WEB_FRAME (web_frame))
                 {
                     JSContextRef js_context = webkit_web_frame_get_global_context (web_frame);
@@ -182,14 +186,14 @@ addons_install_response (GtkWidget*  infobar,
                 }
             }
 
-            if (g_str_has_suffix (uri, ".user.js"))
-                folder = "scripts";
-            else if (g_str_has_suffix (uri, ".user.css"))
-                folder = "styles";
             if (!filename)
                 filename = g_path_get_basename (uri);
-            path = g_build_path (G_DIR_SEPARATOR_S, g_get_user_data_dir (),
-                                 PACKAGE_NAME, folder, filename, NULL);
+            folder_path = g_build_path (G_DIR_SEPARATOR_S, g_get_user_data_dir (),
+                                 PACKAGE_NAME, folder, NULL);
+
+            if (!g_file_test (folder_path, G_FILE_TEST_EXISTS))
+                katze_mkdir_with_parents (folder_path, 0700);
+            path = g_build_path (G_DIR_SEPARATOR_S, folder_path, filename, NULL);
 
             request = webkit_network_request_new (uri);
             download = webkit_download_new (request);
@@ -203,6 +207,7 @@ addons_install_response (GtkWidget*  infobar,
             g_free (path);
             g_free (temp_uri);
             g_free (dest_path);
+            g_free (folder_path);
             g_strfreev (split_uri);
         }
     }
@@ -210,8 +215,7 @@ addons_install_response (GtkWidget*  infobar,
 }
 
 static void
-addons_uri_install (MidoriBrowser* browser,
-                    MidoriView*    view,
+addons_uri_install (MidoriView*    view,
                     AddonsKind     kind)
 {
     const gchar* message;
@@ -239,31 +243,35 @@ addons_uri_install (MidoriBrowser* browser,
 }
 
 static void
-addons_notify_load_status_cb (MidoriBrowser*   browser,
+addons_notify_load_status_cb (MidoriView*      view,
                               GParamSpec*      pspec,
                               MidoriExtension* extension)
 {
-    const gchar* uri = midori_browser_get_current_uri (browser);
+    const gchar* uri = midori_view_get_display_uri (view);
+    WebKitWebView* web_view = WEBKIT_WEB_VIEW (midori_view_get_web_view (view));
+
+    #if WEBKIT_CHECK_VERSION (1, 1, 14)
+    if (webkit_web_view_get_view_source_mode (web_view))
+        return;
+    #endif
 
     if (uri && *uri)
     {
-       /* FIXME: addons_notify_load_status_cb should pass MidoriView* pointer */
-       GtkWidget* view = midori_browser_get_current_tab (browser);
-       if (midori_view_get_load_status (MIDORI_VIEW (view)) == MIDORI_LOAD_FINISHED)
+       if (midori_view_get_load_status (view) == MIDORI_LOAD_COMMITTED)
        {
            /* casual sites goes by uri suffix */
            if (g_str_has_suffix (uri, ".user.js"))
-               addons_uri_install (browser, MIDORI_VIEW (view), ADDONS_USER_SCRIPTS);
+               addons_uri_install (view, ADDONS_USER_SCRIPTS);
            else if (g_str_has_suffix (uri, ".user.css"))
-               addons_uri_install (browser, MIDORI_VIEW (view), ADDONS_USER_STYLES);
+               addons_uri_install (view, ADDONS_USER_STYLES);
            else if (g_str_has_prefix (uri, "http://userscripts.org/scripts/"))
            {
                gchar** split_uri = g_strsplit (uri, "/", -1);
                gchar* subpage = split_uri[4];
 
                /* userscripts.org script main (with desc) and "source view" pages */
-               if (!g_strcmp0 (subpage, "show") /* || !g_strcmp0 (subpage, "review") */)
-                   addons_uri_install (browser, MIDORI_VIEW (view), ADDONS_USER_SCRIPTS);
+               if (!g_strcmp0 (subpage, "show") || !g_strcmp0 (subpage, "review"))
+                   addons_uri_install (view, ADDONS_USER_SCRIPTS);
 
                g_strfreev (split_uri);
            }
@@ -278,7 +286,7 @@ addons_notify_load_status_cb (MidoriBrowser*   browser,
                    subpage = split_uri[5];
                /* userstyles.org style main page with style description */
                if (!subpage)
-                   addons_uri_install (browser, MIDORI_VIEW (view), ADDONS_USER_STYLES);
+                   addons_uri_install (view, ADDONS_USER_STYLES);
 
                g_strfreev (split_uri);
            }
@@ -415,6 +423,7 @@ midori_addons_button_delete_clicked_cb (GtkWidget* toolitem,
         struct AddonElement* element;
         gint delete_response;
         GtkWidget* dialog;
+        gchar* markup;
 
         gtk_tree_model_get (model, &iter, 0, &element, -1);
         dialog = gtk_message_dialog_new (
@@ -431,10 +440,12 @@ midori_addons_button_delete_clicked_cb (GtkWidget* toolitem,
             ? _("Delete user script")
             : _("Delete user style"));
 
-        gtk_message_dialog_format_secondary_markup (
-            GTK_MESSAGE_DIALOG (dialog),
+        markup = g_markup_printf_escaped (
             _("The file <b>%s</b> will be permanently deleted."),
             element->fullpath);
+        gtk_message_dialog_format_secondary_markup (
+            GTK_MESSAGE_DIALOG (dialog), "%s", markup);
+        g_free (markup);
 
         delete_response = gtk_dialog_run (GTK_DIALOG (dialog));
         gtk_widget_destroy (GTK_WIDGET (dialog));
@@ -929,14 +940,15 @@ js_metadata_from_file (const gchar* filename,
     while (g_io_channel_read_line (channel, &line, NULL, NULL, NULL)
            == G_IO_STATUS_NORMAL)
     {
+        g_strstrip (line);
         if (g_str_has_prefix (line, "// ==UserScript=="))
             found_meta = TRUE;
         else if (found_meta)
         {
             if (g_str_has_prefix (line, "// ==/UserScript=="))
                 found_meta = FALSE;
-            else if (g_str_has_prefix (line, "// @require ") ||
-                g_str_has_prefix (line, "// @resource "))
+            else if (g_str_has_prefix (line, "// @require")
+                  || g_str_has_prefix (line, "// @resource"))
             {
                 /* We don't support these, so abort here */
                 g_free (line);
@@ -947,27 +959,29 @@ js_metadata_from_file (const gchar* filename,
                 *excludes = NULL;
                 return FALSE;
             }
-             else if (includes && g_str_has_prefix (line, "// @include "))
+             else if (includes && g_str_has_prefix (line, "// @include"))
             {
-                 rest_of_line = g_strdup (line + strlen ("// @include "));
+                 rest_of_line = g_strdup (line + strlen ("// @include"));
                  rest_of_line =  g_strstrip (rest_of_line);
                  *includes = g_slist_prepend (*includes, rest_of_line);
             }
-             else if (excludes && g_str_has_prefix (line, "// @exclude "))
+             else if (excludes && g_str_has_prefix (line, "// @exclude"))
             {
-                 rest_of_line = g_strdup (line + strlen ("// @exclude "));
+                 rest_of_line = g_strdup (line + strlen ("// @exclude"));
                  rest_of_line =  g_strstrip (rest_of_line);
                  *excludes = g_slist_prepend (*excludes, rest_of_line);
             }
-             else if (name && g_str_has_prefix (line, "// @name "))
+             else if (name && g_str_has_prefix (line, "// @name"))
             {
-                 rest_of_line = g_strdup (line + strlen ("// @name "));
+                 if (!strncmp (line, "// @namespace", 13))
+                     continue;
+                 rest_of_line = g_strdup (line + strlen ("// @name"));
                  rest_of_line =  g_strstrip (rest_of_line);
                  *name = rest_of_line;
             }
-             else if (description && g_str_has_prefix (line, "// @description "))
+             else if (description && g_str_has_prefix (line, "// @description"))
             {
-                 rest_of_line = g_strdup (line + strlen ("// @description "));
+                 rest_of_line = g_strdup (line + strlen ("// @description"));
                  rest_of_line =  g_strstrip (rest_of_line);
                  *description = rest_of_line;
             }
@@ -1001,6 +1015,7 @@ css_metadata_from_file (const gchar* filename,
     while (g_io_channel_read_line (channel, &line, NULL, NULL, NULL)
            == G_IO_STATUS_NORMAL)
     {
+        g_strstrip (line);
         if (g_str_has_prefix (line, "@-moz-document") || line_has_meta)
         { /* FIXME: We merely look for includes. We should honor blocks. */
              if (includes)
@@ -1265,13 +1280,11 @@ addons_update_elements (MidoriExtension* extension,
 
         filename = g_path_get_basename (element->fullpath);
         if (element->description)
-        {
-            tooltip = g_strdup_printf ("%s\n\n%s",
+            tooltip = g_markup_printf_escaped ("%s\n\n%s",
                                        filename, element->description);
-            g_free (filename);
-        }
         else
-            tooltip = filename;
+            tooltip = g_markup_escape_text (filename, -1);
+        g_free (filename);
 
         gtk_list_store_append (liststore, &iter);
         gtk_list_store_set (liststore, &iter,
@@ -1504,6 +1517,8 @@ addons_add_tab_cb (MidoriBrowser* browser,
     GtkWidget* web_view = midori_view_get_web_view (view);
     g_signal_connect (web_view, "window-object-cleared",
         G_CALLBACK (addons_context_ready_cb), extension);
+    g_signal_connect (view, "notify::load-status",
+        G_CALLBACK (addons_notify_load_status_cb), extension);
 }
 
 static void
@@ -1575,8 +1590,6 @@ addons_app_add_browser_cb (MidoriApp*       app,
           (GtkCallback)addons_add_tab_foreach_cb, extension);
     g_signal_connect (browser, "add-tab",
         G_CALLBACK (addons_add_tab_cb), extension);
-    g_signal_connect (browser, "notify::load-status",
-        G_CALLBACK (addons_notify_load_status_cb), extension);
     panel = katze_object_get_object (browser, "panel");
 
     scripts = addons_new (ADDONS_USER_SCRIPTS, extension);
