@@ -10,6 +10,7 @@
 */
 
 #include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
 
 #include <midori/midori.h>
 #include <midori/gtkiconentry.h>
@@ -28,22 +29,17 @@ typedef struct _CookieManagerPagePrivate			CookieManagerPagePrivate;
 
 #define CM_EMPTY_LABEL_TEXT "\n\n\n\n\n\n"
 
-enum
-{
-	COOKIE_MANAGER_COL_NAME,
-	COOKIE_MANAGER_COL_COOKIE,
-	COOKIE_MANAGER_COL_VISIBLE,
-	COOKIE_MANAGER_N_COLUMNS
-};
-
 
 struct _CookieManagerPagePrivate
 {
+	CookieManager *parent;
+
 	GtkWidget *treeview;
 	GtkTreeStore *store;
 	GtkTreeModel *filter;
 
 	GtkWidget *filter_entry;
+	gboolean ignore_changed_filter;
 
 	GtkWidget *desc_label;
 	GtkWidget *delete_button;
@@ -53,11 +49,13 @@ struct _CookieManagerPagePrivate
 
 	GtkWidget *toolbar;
 	GtkWidget *popup_menu;
+};
 
-	GSList *cookies;
-	SoupCookieJar *jar;
-	guint timer_id;
-	gint ignore_changed_count;
+enum
+{
+	PROP_0,
+	PROP_STORE,
+	PROP_PARENT
 };
 
 
@@ -69,7 +67,8 @@ static void cm_button_delete_all_clicked_cb(GtkToolButton *button, CookieManager
 static void cm_tree_popup_collapse_activate_cb(GtkMenuItem *item, CookieManagerPage *cmp);
 static void cm_tree_popup_expand_activate_cb(GtkMenuItem *item, CookieManagerPage *cmp);
 static void cm_filter_tree(CookieManagerPage *cmp, const gchar *filter_text);
-static void cm_jar_changed_cb(SoupCookieJar *jar, SoupCookie *old, SoupCookie *new, CookieManagerPage *cmp);
+
+typedef void (*CMPathWalkFunc) (GtkTreePath *path);
 
 
 G_DEFINE_TYPE_WITH_CODE(CookieManagerPage, cookie_manager_page, GTK_TYPE_VBOX,
@@ -86,23 +85,7 @@ static const gchar *cookie_manager_page_get_label(MidoriViewable *viewable)
 
 static const gchar *cookie_manager_page_get_stock_id(MidoriViewable *viewable)
 {
-	return STOCK_COOKIE_MANAGER;
-}
-
-
-static void cm_free_cookie_list(CookieManagerPage *cmp)
-{
-	CookieManagerPagePrivate *priv = COOKIE_MANAGER_PAGE_GET_PRIVATE(cmp);
-
-	if (priv->cookies != NULL)
-	{
-		GSList *l;
-
-		for (l = priv->cookies; l != NULL; l = g_slist_next(l))
-			soup_cookie_free(l->data);
-		g_slist_free(priv->cookies);
-		priv->cookies = NULL;
-	}
+	return GTK_STOCK_DIALOG_AUTHENTICATION;
 }
 
 
@@ -174,73 +157,19 @@ static void cookie_manager_page_viewable_iface_init(MidoriViewableIface* iface)
 }
 
 
-static void cookie_manager_page_finalize(GObject *object)
+static void cookie_manager_page_pre_cookies_change_cb(CookieManager *cm, CookieManagerPage *cmp)
 {
-	CookieManagerPagePrivate *priv = COOKIE_MANAGER_PAGE_GET_PRIVATE(object);
-
-	g_signal_handlers_disconnect_by_func(priv->jar, cm_jar_changed_cb, object);
-
-	if (priv->timer_id > 0)
-		g_source_remove(priv->timer_id);
-
-	cm_free_cookie_list(COOKIE_MANAGER_PAGE(object));
-
-	gtk_widget_destroy(priv->popup_menu);
-
-	G_OBJECT_CLASS(cookie_manager_page_parent_class)->finalize(object);
-}
-
-
-static void cm_refresh_store(CookieManagerPage *cmp)
-{
-	GSList *l;
-	GHashTable *parents;
-	GtkTreeIter iter;
-	GtkTreeIter *parent_iter;
-	SoupCookie *cookie;
-	const gchar *filter_text;
 	CookieManagerPagePrivate *priv = COOKIE_MANAGER_PAGE_GET_PRIVATE(cmp);
 
 	g_object_ref(priv->filter);
 	gtk_tree_view_set_model(GTK_TREE_VIEW(priv->treeview), NULL);
+}
 
-	gtk_tree_store_clear(priv->store);
 
-	/* free the old list */
-	cm_free_cookie_list(cmp);
-
-	priv->cookies = soup_cookie_jar_all_cookies(priv->jar);
-
-	/* Hashtable holds domain names as keys, the corresponding tree iters as values */
-	parents = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
-
-	for (l = priv->cookies; l != NULL; l = g_slist_next(l))
-	{
-		cookie = l->data;
-
-		/* look for the parent item for the current domain name and create it if it doesn't exist */
-		if ((parent_iter = (GtkTreeIter*) g_hash_table_lookup(parents, cookie->domain)) == NULL)
-		{
-			parent_iter = g_new0(GtkTreeIter, 1);
-
-			gtk_tree_store_append(priv->store, parent_iter, NULL);
-			gtk_tree_store_set(priv->store, parent_iter,
-				COOKIE_MANAGER_COL_NAME, cookie->domain,
-				COOKIE_MANAGER_COL_COOKIE, NULL,
-				COOKIE_MANAGER_COL_VISIBLE, TRUE,
-				-1);
-
-			g_hash_table_insert(parents, g_strdup(cookie->domain), parent_iter);
-		}
-
-		gtk_tree_store_append(priv->store, &iter, parent_iter);
-		gtk_tree_store_set(priv->store, &iter,
-			COOKIE_MANAGER_COL_NAME, cookie->name,
-			COOKIE_MANAGER_COL_COOKIE, cookie,
-			COOKIE_MANAGER_COL_VISIBLE, TRUE,
-			-1);
-	}
-	g_hash_table_destroy(parents);
+static void cookie_manager_page_cookies_changed_cb(CookieManager *cm, CookieManagerPage *cmp)
+{
+	const gchar *filter_text;
+	CookieManagerPagePrivate *priv = COOKIE_MANAGER_PAGE_GET_PRIVATE(cmp);
 
 	gtk_tree_view_set_model(GTK_TREE_VIEW(priv->treeview), GTK_TREE_MODEL(priv->filter));
 	g_object_unref(priv->filter);
@@ -255,32 +184,77 @@ static void cm_refresh_store(CookieManagerPage *cmp)
 }
 
 
-static gboolean cm_delayed_refresh(CookieManagerPage *cmp)
+static void cookie_manager_page_filter_changed_cb(CookieManager *cm, const gchar *text,
+												  CookieManagerPage *cmp)
 {
 	CookieManagerPagePrivate *priv = COOKIE_MANAGER_PAGE_GET_PRIVATE(cmp);
 
-	cm_refresh_store(cmp);
-	priv->timer_id = 0;
-
-	return FALSE;
+	priv->ignore_changed_filter = TRUE;
+	gtk_entry_set_text(GTK_ENTRY(priv->filter_entry), text);
+	priv->ignore_changed_filter = FALSE;
 }
 
 
-static void cm_jar_changed_cb(SoupCookieJar *jar, SoupCookie *old, SoupCookie *new,
-							  CookieManagerPage *cmp)
+static void cookie_manager_page_finalize(GObject *object)
 {
-	CookieManagerPagePrivate *priv = COOKIE_MANAGER_PAGE_GET_PRIVATE(cmp);
+	CookieManagerPagePrivate *priv = COOKIE_MANAGER_PAGE_GET_PRIVATE(object);
 
-	if (priv->ignore_changed_count > 0)
+	gtk_widget_destroy(priv->popup_menu);
+
+	g_signal_handlers_disconnect_by_func(priv->parent,
+		cookie_manager_page_pre_cookies_change_cb, object);
+	g_signal_handlers_disconnect_by_func(priv->parent,
+		cookie_manager_page_cookies_changed_cb, object);
+	g_signal_handlers_disconnect_by_func(priv->parent,
+		cookie_manager_page_filter_changed_cb, object);
+
+	G_OBJECT_CLASS(cookie_manager_page_parent_class)->finalize(object);
+}
+
+
+static void cookie_manager_page_set_property(GObject *object, guint prop_id, const GValue *value,
+											 GParamSpec *pspec)
+{
+	CookieManagerPagePrivate *priv = COOKIE_MANAGER_PAGE_GET_PRIVATE(object);
+	switch (prop_id)
 	{
-		priv->ignore_changed_count--;
-		return;
-	}
+		case PROP_STORE:
+		{
+			priv->store = g_value_get_object(value);
 
-	/* We delay these events a little bit to avoid too many rebuilds of the tree.
-	 * Some websites (like Flyspray bugtrackers sent a whole bunch of cookies at once. */
-	if (priv->timer_id == 0)
-		priv->timer_id = g_timeout_add_seconds(1, (GSourceFunc) cm_delayed_refresh, cmp);
+			/* setting filter and model */
+			priv->filter = gtk_tree_model_filter_new(GTK_TREE_MODEL(priv->store), NULL);
+			gtk_tree_model_filter_set_visible_column(GTK_TREE_MODEL_FILTER(priv->filter),
+				COOKIE_MANAGER_COL_VISIBLE);
+			gtk_tree_view_set_model(GTK_TREE_VIEW(priv->treeview), GTK_TREE_MODEL(priv->filter));
+			g_object_unref(priv->filter);
+			break;
+		}
+		case PROP_PARENT:
+		{
+			if (priv->parent != NULL)
+			{
+				g_signal_handlers_disconnect_by_func(priv->parent,
+					cookie_manager_page_pre_cookies_change_cb, object);
+				g_signal_handlers_disconnect_by_func(priv->parent,
+					cookie_manager_page_cookies_changed_cb, object);
+				g_signal_handlers_disconnect_by_func(priv->parent,
+					cookie_manager_page_filter_changed_cb, object);
+			}
+			priv->parent = g_value_get_object(value);
+
+			g_signal_connect(priv->parent, "pre-cookies-change",
+				G_CALLBACK(cookie_manager_page_pre_cookies_change_cb), object);
+			g_signal_connect(priv->parent, "cookies-changed",
+				G_CALLBACK(cookie_manager_page_cookies_changed_cb), object);
+			g_signal_connect(priv->parent, "filter-changed",
+				G_CALLBACK(cookie_manager_page_filter_changed_cb), object);
+			break;
+		}
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
+			break;
+	}
 }
 
 
@@ -288,7 +262,27 @@ static void cookie_manager_page_class_init(CookieManagerPageClass *klass)
 {
 	GObjectClass *g_object_class;
 	g_object_class = G_OBJECT_CLASS(klass);
+
 	g_object_class->finalize = cookie_manager_page_finalize;
+	g_object_class->set_property = cookie_manager_page_set_property;
+
+	g_object_class_install_property(g_object_class,
+		PROP_STORE,
+		g_param_spec_object(
+		"store",
+		"Treestore",
+		"The tree store",
+		GTK_TYPE_TREE_STORE,
+		G_PARAM_WRITABLE));
+
+	g_object_class_install_property(g_object_class,
+		PROP_PARENT,
+		g_param_spec_object(
+		"parent",
+		"Parent",
+		"The CookieManager parent instance",
+		COOKIE_MANAGER_TYPE,
+		G_PARAM_WRITABLE));
 
 	g_type_class_add_private(klass, sizeof(CookieManagerPagePrivate));
 }
@@ -312,6 +306,28 @@ static void cm_set_button_sensitiveness(CookieManagerPage *cmp, gboolean set)
 }
 
 
+static void cm_free_selection_list(GList *rows, GFunc func)
+{
+	g_list_foreach(rows, func, NULL);
+	g_list_free(rows);
+}
+
+
+/* Fast version of g_list_length(). It only checks for the first few elements of
+ * the list and returns the length 0, 1 or 2 where 2 means 2 elements or more. */
+static gint cm_list_length(GList *list)
+{
+	if (list == NULL)
+		return 0;
+	else if (list->next == NULL)
+		return 1;
+	else if (list->next != NULL)
+		return 2;
+
+	return 0; /* safe default */
+}
+
+
 static void cm_tree_popup_collapse_activate_cb(GtkMenuItem *item, CookieManagerPage *cmp)
 {
 	CookieManagerPagePrivate *priv = COOKIE_MANAGER_PAGE_GET_PRIVATE(cmp);
@@ -346,82 +362,169 @@ static void cm_delete_cookie(CookieManagerPage *cmp, GtkTreeModel *model, GtkTre
 
 	gtk_tree_model_get(model, child, COOKIE_MANAGER_COL_COOKIE, &cookie, -1);
 
-	if (cookie != NULL)
-	{
-		priv->ignore_changed_count++;
+	cookie_manager_delete_cookie(priv->parent, cookie);
+}
 
-		soup_cookie_jar_delete_cookie(priv->jar, cookie);
-		/* the SoupCookie object is freed when the whole list gets updated */
+
+static gboolean cm_try_to_select(CMPathWalkFunc path_func, GtkTreeSelection *selection,
+								 GtkTreeModel *model, GtkTreePath *path)
+{
+	GtkTreeIter iter;
+
+	if (gtk_tree_path_get_depth(path) <= 0) /* sanity check */
+		return FALSE;
+
+	/* modify the path using the passed function */
+	if (path_func != NULL)
+		path_func(path);
+
+	if (gtk_tree_path_get_depth(path) <= 0) /* sanity check */
+		return FALSE;
+
+	/* check whether the path points to something valid and if so, select it */
+	if (gtk_tree_model_get_iter(model, &iter, path))
+	{
+		GtkTreeView *treeview = gtk_tree_selection_get_tree_view(selection);
+		gboolean was_expanded = gtk_tree_view_row_expanded(treeview, path);
+		/* to get gtk_tree_selection_select_path() working, we need to expand the row first
+		 * if it isn't expanded yet, at least when the row is a parent item */
+		if (! was_expanded)
+			gtk_tree_view_expand_to_path(treeview, path);
+
+		gtk_tree_selection_select_path(selection, path);
+
+		if (! was_expanded) /* restore the previous state */
+			gtk_tree_view_collapse_row(treeview, path);
+		return TRUE;
 	}
+
+	return FALSE;
+}
+
+
+/* select an item after deletion */
+static void cm_select_path(CookieManagerPage *cmp, GtkTreeModel *model, GtkTreePath *path)
+{
+	CookieManagerPagePrivate *priv = COOKIE_MANAGER_PAGE_GET_PRIVATE(cmp);
+	GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(priv->treeview));
+	CMPathWalkFunc path_funcs[] = {
+		(CMPathWalkFunc) gtk_tree_path_prev, (CMPathWalkFunc) gtk_tree_path_up,
+		(CMPathWalkFunc) gtk_tree_path_next, NULL };
+	CMPathWalkFunc *path_func;
+
+	/* first try selecting the item directly to which path points */
+	if (! cm_try_to_select(NULL, selection, model, path))
+	{	/* if this failed, modify the path until we found something valid */
+		path_func = path_funcs;
+		while (*path_func != NULL)
+		{
+			if (cm_try_to_select(*path_func, selection, model, path))
+				break;
+			path_func++;
+		}
+	}
+}
+
+
+static void cm_delete_item(CookieManagerPage *cmp)
+{
+	GtkTreeIter iter, iter_store, child;
+	GtkTreeModel *model;
+	GtkTreePath *path, *last_path;
+	GtkTreeSelection *selection;
+	GList *rows, *row;
+	GList *refs = NULL;
+	CookieManagerPagePrivate *priv = COOKIE_MANAGER_PAGE_GET_PRIVATE(cmp);
+
+	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(priv->treeview));
+	rows = gtk_tree_selection_get_selected_rows(selection, &model);
+	if (cm_list_length(rows) == 0)
+		return;
+
+	last_path = gtk_tree_path_copy(g_list_nth_data(rows, 0));
+
+	/* as paths will change during delete, first create GtkTreeRowReferences for
+	 * all selected rows */
+	row = rows;
+	do
+	{
+		refs = g_list_append(refs, gtk_tree_row_reference_new(model, (GtkTreePath*) (row->data)));
+	} while ((row = row->next) != NULL);
+
+	row = refs;
+	do
+	{
+		/* get iter */
+		path = gtk_tree_row_reference_get_path((GtkTreeRowReference*) row->data);
+		if (path == NULL)
+			continue;
+		gtk_tree_model_get_iter(model, &iter, path);
+
+		if (gtk_tree_model_iter_has_child(model, &iter))
+		{
+			while (gtk_tree_model_iter_children(model, &child, &iter))
+			{
+				cm_delete_cookie(cmp, model, &child);
+				cm_store_remove(cmp, &child);
+				/* we retrieve again the iter at path because it got invalid by the delete operation */
+				gtk_tree_model_get_iter(model, &iter, path);
+			}
+			/* remove/hide the parent */
+			gtk_tree_model_filter_convert_iter_to_child_iter(GTK_TREE_MODEL_FILTER(priv->filter),
+				&iter_store, &iter);
+			if (gtk_tree_model_iter_has_child(GTK_TREE_MODEL(priv->store), &iter_store))
+				gtk_tree_store_set(priv->store, &iter_store, COOKIE_MANAGER_COL_VISIBLE, FALSE, -1);
+			else
+				cm_store_remove(cmp, &iter);
+		}
+		else
+		{
+			GtkTreePath *path_store, *path_model;
+
+			gtk_tree_model_filter_convert_iter_to_child_iter(GTK_TREE_MODEL_FILTER(priv->filter),
+				&iter_store, &iter);
+			path_store = gtk_tree_model_get_path(GTK_TREE_MODEL(priv->store), &iter_store);
+			path_model = gtk_tree_model_get_path(model, &iter);
+
+			cm_delete_cookie(cmp, model, &iter);
+			gtk_tree_store_remove(priv->store, &iter_store);
+
+			/* check whether the parent still has children, otherwise delete it */
+			if (gtk_tree_path_up(path_store))
+			{
+				gtk_tree_model_get_iter(GTK_TREE_MODEL(priv->store), &iter_store, path_store);
+				if (! gtk_tree_model_iter_has_child(GTK_TREE_MODEL(priv->store), &iter_store))
+					/* remove the empty parent */
+					gtk_tree_store_remove(priv->store, &iter_store);
+			}
+			/* now for the filter model */
+			if (gtk_tree_path_up(path_model))
+			{
+				gtk_tree_model_get_iter(model, &iter, path_model);
+				if (! gtk_tree_model_iter_has_child(model, &iter))
+				{
+					gtk_tree_model_filter_convert_iter_to_child_iter(
+						GTK_TREE_MODEL_FILTER(priv->filter), &iter_store, &iter);
+					/* hide the empty parent */
+					gtk_tree_store_set(priv->store, &iter_store, COOKIE_MANAGER_COL_VISIBLE, FALSE, -1);
+				}
+			}
+			gtk_tree_path_free(path_store);
+			gtk_tree_path_free(path_model);
+		}
+		gtk_tree_path_free(path);
+	} while ((row = row->next) != NULL);
+	cm_free_selection_list(rows, (GFunc) gtk_tree_path_free);
+	cm_free_selection_list(refs, (GFunc) gtk_tree_row_reference_free);
+
+	cm_select_path(cmp, model, last_path);
+	gtk_tree_path_free(last_path);
 }
 
 
 static void cm_button_delete_clicked_cb(GtkToolButton *button, CookieManagerPage *cmp)
 {
-	GtkTreeIter iter, iter_store, child;
-	GtkTreeModel *model;
-	GtkTreeSelection *selection;
-	CookieManagerPagePrivate *priv = COOKIE_MANAGER_PAGE_GET_PRIVATE(cmp);
-
-	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(priv->treeview));
-	if (! gtk_tree_selection_get_selected(selection, &model, &iter))
-		return;
-
-	if (gtk_tree_model_iter_has_child(model, &iter))
-	{
-		GtkTreePath *path = gtk_tree_model_get_path(model, &iter);
-
-		while (gtk_tree_model_iter_children(model, &child, &iter))
-		{
-			cm_delete_cookie(cmp, model, &child);
-			cm_store_remove(cmp, &child);
-			/* we retrieve again the iter at path because it got invalid by the delete operation */
-			gtk_tree_model_get_iter(model, &iter, path);
-		}
-		gtk_tree_path_free(path);
-		/* remove/hide the parent */
-		gtk_tree_model_filter_convert_iter_to_child_iter(GTK_TREE_MODEL_FILTER(priv->filter),
-			&iter_store, &iter);
-		if (gtk_tree_model_iter_has_child(GTK_TREE_MODEL(priv->store), &iter_store))
-			gtk_tree_store_set(priv->store, &iter_store, COOKIE_MANAGER_COL_VISIBLE, FALSE, -1);
-		else
-			cm_store_remove(cmp, &iter);
-	}
-	else
-	{
-		GtkTreePath *path_store, *path_model;
-
-		gtk_tree_model_filter_convert_iter_to_child_iter(GTK_TREE_MODEL_FILTER(priv->filter),
-			&iter_store, &iter);
-		path_store = gtk_tree_model_get_path(GTK_TREE_MODEL(priv->store), &iter_store);
-		path_model = gtk_tree_model_get_path(model, &iter);
-
-		cm_delete_cookie(cmp, model, &iter);
-		gtk_tree_store_remove(priv->store, &iter_store);
-
-		/* check whether the parent still has children, otherwise delete it */
-		if (gtk_tree_path_up(path_store))
-		{
-			gtk_tree_model_get_iter(GTK_TREE_MODEL(priv->store), &iter_store, path_store);
-			if (! gtk_tree_model_iter_has_child(GTK_TREE_MODEL(priv->store), &iter_store))
-				/* remove the empty parent */
-				gtk_tree_store_remove(priv->store, &iter_store);
-		}
-		/* now for the filter model */
-		if (gtk_tree_path_up(path_model))
-		{
-			gtk_tree_model_get_iter(model, &iter, path_model);
-			if (! gtk_tree_model_iter_has_child(model, &iter))
-			{
-				gtk_tree_model_filter_convert_iter_to_child_iter(
-					GTK_TREE_MODEL_FILTER(priv->filter), &iter_store, &iter);
-				/* hide the empty parent */
-				gtk_tree_store_set(priv->store, &iter_store, COOKIE_MANAGER_COL_VISIBLE, FALSE, -1);
-			}
-		}
-		gtk_tree_path_free(path_store);
-		gtk_tree_path_free(path_model);
-	}
+	cm_delete_item(cmp);
 }
 
 
@@ -452,11 +555,12 @@ static void cm_delete_all_cookies_real(CookieManagerPage *cmp)
 		else
 			cm_store_remove(cmp, &iter);
 	}
-	gtk_tree_path_free(path_first);
-
 	/* now that we deleted all matching cookies, we reset the filter */
 	gtk_entry_set_text(GTK_ENTRY(priv->filter_entry), "");
 	cm_set_button_sensitiveness(cmp, FALSE);
+
+	cm_select_path(cmp, model, path_first);
+	gtk_tree_path_free(path_first);
 }
 
 
@@ -492,6 +596,12 @@ static void cm_button_delete_all_clicked_cb(GtkToolButton *button, CookieManager
 }
 
 
+static const gchar *cm_skip_leading_dot(const gchar *text)
+{
+	return (*text == '.') ? text + 1 : text;
+}
+
+
 static void cm_tree_drag_data_get_cb(GtkWidget *widget, GdkDragContext *drag_context,
 									 GtkSelectionData *data, guint info, guint ltime,
 									 CookieManagerPage *cmp)
@@ -499,11 +609,18 @@ static void cm_tree_drag_data_get_cb(GtkWidget *widget, GdkDragContext *drag_con
 	GtkTreeIter iter, iter_store;
 	GtkTreeSelection *selection;
 	GtkTreeModel *model;
+	GList *rows;
 	CookieManagerPagePrivate *priv = COOKIE_MANAGER_PAGE_GET_PRIVATE(cmp);
 
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(priv->treeview));
-	if (! gtk_tree_selection_get_selected(selection, &model, &iter))
+	rows = gtk_tree_selection_get_selected_rows(selection, &model);
+	if (cm_list_length(rows) != 1)
+	{
+		cm_free_selection_list(rows, (GFunc) gtk_tree_path_free);
 		return;
+	}
+	/* get iter */
+	gtk_tree_model_get_iter(model, &iter, (GtkTreePath*) (g_list_nth_data(rows, 0)));
 
 	gtk_tree_model_filter_convert_iter_to_child_iter(
 		GTK_TREE_MODEL_FILTER(model), &iter_store, &iter);
@@ -511,18 +628,25 @@ static void cm_tree_drag_data_get_cb(GtkWidget *widget, GdkDragContext *drag_con
 	if (gtk_tree_store_iter_is_valid(priv->store, &iter_store))
 	{
 		SoupCookie *cookie;
-		gchar *name, *text;
+		gchar *name;
+		const gchar *text;
 
 		gtk_tree_model_get(model, &iter,
 			COOKIE_MANAGER_COL_NAME, &name,
 			COOKIE_MANAGER_COL_COOKIE, &cookie,
 			-1);
 
-		if (cookie == NULL && name != NULL)
+		if (name != NULL)
 		{
-			/* skip a leading dot */
-			text = (*name == '.') ? name + 1 : name;
+			GtkTreeIter parent;
+			/* get the name of the parent item which should be a domain item */
+			if (cookie != NULL && gtk_tree_model_iter_parent(model, &parent, &iter))
+			{
+				g_free(name);
+				gtk_tree_model_get(model, &parent, COOKIE_MANAGER_COL_NAME, &name, -1);
+			}
 
+			text = cm_skip_leading_dot(name);
 			gtk_selection_data_set_text(data, text, -1);
 		}
 		g_free(name);
@@ -566,6 +690,24 @@ static gchar *cm_get_cookie_description_text(SoupCookie *cookie)
 }
 
 
+static gchar *cm_get_domain_description_text(const gchar *domain, gint cookie_count)
+{
+	gchar *str, *text;
+
+	domain = cm_skip_leading_dot(domain);
+
+	text = g_markup_printf_escaped(
+		_("<b>Domain</b>: %s\n<b>Cookies</b>: %d"),
+		domain, cookie_count);
+
+	str = g_strconcat(text, "\n\n\n\n", NULL);
+
+	g_free(text);
+
+	return str;
+}
+
+
 #if GTK_CHECK_VERSION(2, 12, 0)
 static gboolean cm_tree_query_tooltip(GtkWidget *widget, gint x, gint y, gboolean keyboard_mode,
 									  GtkTooltip *tooltip, CookieManagerPage *cmp)
@@ -670,11 +812,17 @@ static void cm_filter_tree(CookieManagerPage *cmp, const gchar *filter_text)
 
 static void cm_filter_entry_changed_cb(GtkEditable *editable, CookieManagerPage *cmp)
 {
-	const gchar *text = gtk_entry_get_text(GTK_ENTRY(editable));
+	const gchar *text;
 	CookieManagerPagePrivate *priv = COOKIE_MANAGER_PAGE_GET_PRIVATE(cmp);
 
+	if (priv->ignore_changed_filter)
+		return;
+
+	text = gtk_entry_get_text(GTK_ENTRY(editable));
 	cm_filter_tree(cmp, text);
 
+	cookie_manager_update_filter(priv->parent, text);
+
 	if (*text != '\0')
 		gtk_tree_view_expand_all(GTK_TREE_VIEW(priv->treeview));
 	else
@@ -691,24 +839,36 @@ static void cm_filter_entry_clear_icon_released_cb(GtkIconEntry *e, gint pos, gi
 
 static void cm_tree_selection_changed_cb(GtkTreeSelection *selection, CookieManagerPage *cmp)
 {
+	GList *rows;
 	GtkTreeIter iter, iter_store;
 	GtkTreeModel *model;
-	gchar *text;
+	gchar *text, *name;
 	gboolean valid = TRUE;
-	gboolean delete_possible = FALSE;
+	gboolean delete_possible = TRUE;
+	guint rows_len;
 	SoupCookie *cookie;
 	CookieManagerPagePrivate *priv = COOKIE_MANAGER_PAGE_GET_PRIVATE(cmp);
 
-	if (! gtk_tree_selection_get_selected(selection, &model, &iter))
+	rows = gtk_tree_selection_get_selected_rows(selection, &model);
+	rows_len = cm_list_length(rows);
+	if (rows_len == 0)
+	{
 		valid = FALSE;
-	else
+		delete_possible = FALSE;
+	}
+	else if (rows_len == 1)
+	{
+		/* get iter */
+		gtk_tree_model_get_iter(model, &iter, (GtkTreePath*) (g_list_nth_data(rows, 0)));
+
 		gtk_tree_model_filter_convert_iter_to_child_iter(GTK_TREE_MODEL_FILTER(model),
 			&iter_store, &iter);
+	}
+	else
+		valid = FALSE;
 
 	if (valid && gtk_tree_store_iter_is_valid(priv->store, &iter_store))
 	{
-		delete_possible = TRUE;
-
 		gtk_tree_model_get(model, &iter, COOKIE_MANAGER_COL_COOKIE, &cookie, -1);
 		if (cookie != NULL)
 		{
@@ -719,13 +879,27 @@ static void cm_tree_selection_changed_cb(GtkTreeSelection *selection, CookieMana
 			g_free(text);
 		}
 		else
-			valid = FALSE;
+		{
+			gtk_tree_model_get(model, &iter, COOKIE_MANAGER_COL_NAME, &name, -1);
+			if (name != NULL)
+			{
+				gint cookie_count = gtk_tree_model_iter_n_children(model, &iter);
+
+				text = cm_get_domain_description_text(name, cookie_count);
+				gtk_label_set_markup(GTK_LABEL(priv->desc_label), text);
+
+				g_free(text);
+				g_free(name);
+			}
+		}
 	}
 	/* This is a bit hack'ish but we add some empty lines to get a minimum height of the
 	 * label at the bottom without any font size calculation. */
 	if (! valid)
 		gtk_label_set_text(GTK_LABEL(priv->desc_label), CM_EMPTY_LABEL_TEXT);
 	cm_set_button_sensitiveness(cmp, delete_possible);
+
+	cm_free_selection_list(rows, (GFunc) gtk_tree_path_free);
 }
 
 
@@ -768,19 +942,36 @@ static gboolean cm_tree_button_release_event_cb(GtkWidget *widget, GdkEventButto
 }
 
 
+static gboolean cm_tree_key_press_cb(GtkWidget *widget, GdkEventKey *event, CookieManagerPage *cmp)
+{
+	if (event->keyval == GDK_Delete && !
+		(event->state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK | GDK_MOD1_MASK)))
+	{
+		cm_delete_item(cmp);
+		return TRUE;
+	}
+	return FALSE;
+}
+
+
 static gboolean cm_tree_button_press_event_cb(GtkWidget *widget, GdkEventButton *ev,
 											  CookieManagerPage *cmp)
 {
+	gboolean ret = FALSE;
+
 	if (ev->type == GDK_2BUTTON_PRESS)
 	{
 		GtkTreeSelection *selection;
 		GtkTreeModel *model;
 		GtkTreeIter iter;
+		GList *rows;
 
 		selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
-
-		if (gtk_tree_selection_get_selected(selection, &model, &iter))
+		rows = gtk_tree_selection_get_selected_rows(selection, &model);
+		if (cm_list_length(rows) == 1)
 		{
+			/* get iter */
+			gtk_tree_model_get_iter(model, &iter, (GtkTreePath*) (g_list_nth_data(rows, 0)));
 			/* double click on parent node expands/collapses it */
 			if (gtk_tree_model_iter_has_child(model, &iter))
 			{
@@ -793,12 +984,32 @@ static gboolean cm_tree_button_press_event_cb(GtkWidget *widget, GdkEventButton
 
 				gtk_tree_path_free(path);
 
-				return TRUE;
+				ret = TRUE;
 			}
 		}
+		cm_free_selection_list(rows, (GFunc) gtk_tree_path_free);
 	}
+	return ret;
+}
 
-	return FALSE;
+
+static void cm_tree_render_text_cb(GtkTreeViewColumn *column, GtkCellRenderer *renderer, GtkTreeModel *model,
+								   GtkTreeIter *iter, gpointer data)
+{
+	gchar *name;
+
+	gtk_tree_model_get(model, iter, COOKIE_MANAGER_COL_NAME, &name, -1);
+
+	if (name != NULL && *name != '.')
+	{
+		gchar *display_name = g_strconcat(" ", name, NULL);
+		g_object_set(renderer, "text", display_name, NULL);
+		g_free(display_name);
+	}
+	else
+		g_object_set(renderer, "text", name, NULL);
+
+	g_free(name);
 }
 
 
@@ -820,6 +1031,8 @@ static GtkWidget *cm_tree_prepare(CookieManagerPage *cmp)
 	gtk_tree_view_column_set_sort_indicator(column, TRUE);
 	gtk_tree_view_column_set_sort_column_id(column, COOKIE_MANAGER_COL_NAME);
 	gtk_tree_view_column_set_resizable(column, TRUE);
+	gtk_tree_view_column_set_cell_data_func(column, renderer,
+        (GtkTreeCellDataFunc) cm_tree_render_text_cb, NULL, NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
 
 	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(treeview), TRUE);
@@ -828,24 +1041,11 @@ static GtkWidget *cm_tree_prepare(CookieManagerPage *cmp)
 
 	/* selection handling */
 	sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
-	gtk_tree_selection_set_mode(sel, GTK_SELECTION_SINGLE);
-
-	/* create the main store */
-	priv->store = gtk_tree_store_new(COOKIE_MANAGER_N_COLUMNS,
-		G_TYPE_STRING, SOUP_TYPE_COOKIE, G_TYPE_BOOLEAN);
-	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(priv->store),
-		COOKIE_MANAGER_COL_NAME, GTK_SORT_ASCENDING);
-
-	/* setting filter and model */
-	priv->filter = gtk_tree_model_filter_new(GTK_TREE_MODEL(priv->store), NULL);
-	gtk_tree_model_filter_set_visible_column(GTK_TREE_MODEL_FILTER(priv->filter),
-		COOKIE_MANAGER_COL_VISIBLE);
-	gtk_tree_view_set_model(GTK_TREE_VIEW(priv->treeview), GTK_TREE_MODEL(priv->filter));
-	g_object_unref(priv->store);
-	g_object_unref(priv->filter);
+	gtk_tree_selection_set_mode(sel, GTK_SELECTION_MULTIPLE);
 
 	/* signals */
 	g_signal_connect(sel, "changed", G_CALLBACK(cm_tree_selection_changed_cb), cmp);
+	g_signal_connect(treeview, "key-press-event", G_CALLBACK(cm_tree_key_press_cb), cmp);
 	g_signal_connect(treeview, "button-press-event", G_CALLBACK(cm_tree_button_press_event_cb), cmp);
 	g_signal_connect(treeview, "button-release-event", G_CALLBACK(cm_tree_button_release_event_cb), cmp);
 	g_signal_connect(treeview, "popup-menu", G_CALLBACK(cm_tree_popup_menu_cb), cmp);
@@ -865,7 +1065,6 @@ static GtkWidget *cm_tree_prepare(CookieManagerPage *cmp)
 		GDK_ACTION_COPY
 	);
 	gtk_drag_source_add_text_targets(treeview);
-	/*gtk_drag_source_add_uri_targets(treeview);*/
 	g_signal_connect(treeview, "drag-data-get", G_CALLBACK(cm_tree_drag_data_get_cb), cmp);
 
 	/* popup menu */
@@ -909,9 +1108,12 @@ static void cookie_manager_page_init(CookieManagerPage *self)
 	GtkWidget *filter_hbox;
 	GtkWidget *filter_label;
 	GtkWidget *treeview;
-	SoupSession *session;
 	CookieManagerPagePrivate *priv = COOKIE_MANAGER_PAGE_GET_PRIVATE(self);
 
+	priv->parent = NULL;
+	priv->store = NULL;
+	priv->ignore_changed_filter = FALSE;
+
 	cm_create_toolbar(self);
 
 	priv->desc_label = gtk_label_new(CM_EMPTY_LABEL_TEXT);
@@ -968,18 +1170,19 @@ static void cookie_manager_page_init(CookieManagerPage *self)
 
 	gtk_box_pack_start(GTK_BOX(self), filter_hbox, FALSE, FALSE, 5);
 	gtk_box_pack_start(GTK_BOX(self), paned, TRUE, TRUE, 0);
-
-	/* setup soup */
-	session = webkit_get_default_session();
-	priv->jar = SOUP_COOKIE_JAR(soup_session_get_feature(session, soup_cookie_jar_get_type()));
-	g_signal_connect(priv->jar, "changed", G_CALLBACK(cm_jar_changed_cb), self);
-
-	cm_refresh_store(self);
 }
 
 
-GtkWidget *cookie_manager_page_new(void)
+GtkWidget *cookie_manager_page_new(CookieManager *parent, GtkTreeStore *store,
+								   const gchar *filter_text)
 {
-	return g_object_new(COOKIE_MANAGER_PAGE_TYPE, NULL);
+	GtkWidget *cmp;
+
+	cmp = g_object_new(COOKIE_MANAGER_PAGE_TYPE, "parent", parent, "store", store, NULL);
+
+	if (filter_text != NULL)
+		cookie_manager_page_filter_changed_cb(parent, filter_text, COOKIE_MANAGER_PAGE(cmp));
+
+	return cmp;
 }
 
