@@ -1,5 +1,5 @@
 /*
- Copyright (C) 2008-2010 Christian Dywan <christian@twotoasts.de>
+ Copyright (C) 2008-2011 Christian Dywan <christian@twotoasts.de>
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
@@ -50,6 +50,9 @@ struct _KatzeArrayClass
                                gint        index);
     void
     (*clear)                  (KatzeArray* array);
+
+    void
+    (*update)                 (KatzeArray* array);
 };
 
 G_DEFINE_TYPE (KatzeArray, katze_array, KATZE_TYPE_ITEM);
@@ -59,6 +62,7 @@ enum {
     REMOVE_ITEM,
     MOVE_ITEM,
     CLEAR,
+    UPDATE,
 
     LAST_SIGNAL
 };
@@ -74,15 +78,10 @@ static void
 _katze_array_add_item (KatzeArray* array,
                        gpointer    item)
 {
-    if (katze_array_is_a (array, G_TYPE_OBJECT))
-    {
-        GType type = G_OBJECT_TYPE (item);
-
-        /* g_return_if_fail (katze_array_is_a (array, type)); */
-        g_object_ref (item);
-        if (g_type_is_a (type, KATZE_TYPE_ITEM))
-            katze_item_set_parent (item, array);
-    }
+    GType type = G_OBJECT_TYPE (item);
+    g_object_ref (item);
+    if (g_type_is_a (type, KATZE_TYPE_ITEM))
+        katze_item_set_parent (item, array);
 
     array->items = g_list_append (array->items, item);
 }
@@ -93,12 +92,9 @@ _katze_array_remove_item (KatzeArray* array,
 {
     array->items = g_list_remove (array->items, item);
 
-    if (katze_array_is_a (array, G_TYPE_OBJECT))
-    {
-        if (KATZE_IS_ITEM (item))
-            katze_item_set_parent (item, NULL);
-        g_object_unref (item);
-    }
+    if (KATZE_IS_ITEM (item))
+        katze_item_set_parent (item, NULL);
+    g_object_unref (item);
 }
 
 static void
@@ -183,6 +179,25 @@ katze_array_class_init (KatzeArrayClass* class)
         g_cclosure_marshal_VOID__VOID,
         G_TYPE_NONE, 0);
 
+    /**
+     * KatzeArray::update:
+     * @array: the object on which the signal is emitted
+     *
+     * The array changed and any display widgets should
+     * be updated.
+     *
+     * Since: 0.3.0
+     **/
+    signals[UPDATE] = g_signal_new (
+        "update",
+        G_TYPE_FROM_CLASS (class),
+        (GSignalFlags)(G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION),
+        0,
+        0,
+        NULL,
+        g_cclosure_marshal_VOID__VOID,
+        G_TYPE_NONE, 0);
+
     gobject_class = G_OBJECT_CLASS (class);
     gobject_class->finalize = katze_array_finalize;
 
@@ -195,7 +210,7 @@ katze_array_class_init (KatzeArrayClass* class)
 static void
 katze_array_init (KatzeArray* array)
 {
-    array->type = G_TYPE_NONE;
+    array->type = G_TYPE_OBJECT;
     array->items = NULL;
 }
 
@@ -204,16 +219,12 @@ katze_array_finalize (GObject* object)
 {
     KatzeArray* array;
     guint i;
+    gpointer item;
 
     array = KATZE_ARRAY (object);
-    if (katze_array_is_a (array, G_TYPE_OBJECT))
-    {
-        gpointer item;
-        i = 0;
-        while ((item = g_list_nth_data (array->items, i++)))
-            g_object_unref (item);
-    }
-
+    i = 0;
+    while ((item = g_list_nth_data (array->items, i++)))
+        g_object_unref (item);
     g_list_free (array->items);
 
     G_OBJECT_CLASS (katze_array_parent_class)->finalize (object);
@@ -225,8 +236,6 @@ katze_array_finalize (GObject* object)
  *
  * Creates a new #KatzeArray for @type items.
  *
- * You may only add items of the given type or inherited
- * from it to this array *if* @type is an #GObject type.
  * The array will keep a reference on each object until
  * it is removed from the array.
  *
@@ -235,14 +244,17 @@ katze_array_finalize (GObject* object)
 KatzeArray*
 katze_array_new (GType type)
 {
-    KatzeArray* array = g_object_new (KATZE_TYPE_ARRAY, NULL);
+    KatzeArray* array;
+
+    g_return_val_if_fail (g_type_is_a (type, G_TYPE_OBJECT), NULL);
+
+    array = g_object_new (KATZE_TYPE_ARRAY, NULL);
     array->type = type;
 
     return array;
 }
 
 /**
- *
  * katze_array_is_a:
  * @array: a #KatzeArray
  * @is_a_type: type to compare with
@@ -371,9 +383,6 @@ katze_array_find_token (KatzeArray*  array,
     guint i;
     gpointer item;
 
-    if (!katze_array_is_a (array, G_TYPE_OBJECT))
-        return NULL;
-
     i = 0;
     while ((item = g_list_nth_data (array->items, i++)))
     {
@@ -410,9 +419,6 @@ katze_array_find_uri (KatzeArray*  array,
     guint i;
     gpointer item;
 
-    if (!katze_array_is_a (array, G_TYPE_OBJECT))
-        return NULL;
-
     i = 0;
     while ((item = g_list_nth_data (array->items, i++)))
     {
@@ -502,3 +508,20 @@ katze_array_clear (KatzeArray* array)
 
     g_signal_emit (array, signals[CLEAR], 0);
 }
+
+/**
+ * katze_array_update:
+ * @array: a #KatzeArray
+ *
+ * Indicates that the array changed and any display
+ * widgets should be updated.
+ *
+ * Since: 0.3.0
+ **/
+void
+katze_array_update (KatzeArray* array)
+{
+    g_return_if_fail (KATZE_IS_ARRAY (array));
+
+    g_signal_emit (array, signals[UPDATE], 0);
+}
