@@ -293,6 +293,45 @@ katze_array_from_xmlDocPtr (KatzeArray* array,
     return TRUE;
 }
 
+static gchar*
+katze_unescape_html (const gchar* text)
+{
+    gchar* amp = g_strstr_len (text, -1, "&");
+
+    if (amp && *amp)
+    {
+        if  (!strncmp (amp, "&quot;", 6) || !strncmp (amp, "&amp;", 5)
+          || !strncmp (amp, "&lt;", 4)   || !strncmp (amp, "&gt;", 4)
+          || !strncmp (amp, "&apos;", 6))
+        {
+            guint i = 0;
+            gchar** parts = g_strsplit_set (text, "&;", -1);
+            GString *unescaped = g_string_new (NULL);
+
+            while (parts[i])
+            {
+                if (katze_str_equal ("quot", parts[i]))
+                    g_string_append (unescaped, "\"");
+                else if (katze_str_equal ("amp", parts[i]))
+                    g_string_append (unescaped, "&");
+                else if (katze_str_equal ("lt", parts[i]))
+                    g_string_append (unescaped, "<");
+                else if (katze_str_equal ("gt", parts[i]))
+                    g_string_append (unescaped, ">");
+                else if (katze_str_equal ("apos", parts[i]))
+                    g_string_append (unescaped, "'");
+                else
+                    g_string_append (unescaped, parts[i]);
+                i++;
+            }
+            g_strfreev (parts);
+
+            return g_string_free (unescaped, FALSE);
+        }
+    }
+    return g_strdup (text);
+}
+
 static gboolean
 katze_array_from_netscape_file (KatzeArray* array,
                                const gchar* filename)
@@ -322,8 +361,8 @@ katze_array_from_netscape_file (KatzeArray* array,
                     gchar** parts = g_strsplit (line, "\"", -1);
                     item = katze_item_new ();
                     katze_array_add_item (folder, item);
-                    item->name = g_strdup (element[4]);
-                    item->uri = g_strdup (parts[1]);
+                    item->name = katze_unescape_html (element[4]);
+                    item->uri = katze_unescape_html (parts[1]);
                     g_strfreev (parts);
                 }
                 /* item is folder */
@@ -332,14 +371,14 @@ katze_array_from_netscape_file (KatzeArray* array,
                     item = (KatzeItem*)katze_array_new (KATZE_TYPE_ARRAY);
                     katze_array_add_item (folder, item);
                     folder = (KatzeArray*)item;
-                    item->name = g_strdup (element[4]);
+                    item->name = katze_unescape_html (element[4]);
                 }
             }
             /* item description */
             if (item && katze_str_equal (element[1], "DD"))
             {
                 if (element[2])
-                    item->text = g_strdup (element[2]);
+                    item->text = katze_unescape_html (element[2]);
                 item = NULL;
             }
             /* end of current folder, level-up */
@@ -702,6 +741,48 @@ string_append_item (GString*   string,
     g_free (metadata);
 }
 
+static void
+string_append_netscape_item (GString*   string,
+                             KatzeItem* item)
+{
+    g_return_if_fail (KATZE_IS_ITEM (item));
+
+    if (KATZE_IS_ARRAY (item))
+    {
+        KatzeItem* _item;
+        KatzeArray* array = KATZE_ARRAY (item);
+        GList* list;
+
+        g_string_append (string, "\t<DT><H3 FOLDED ADD_DATE=\"\">");
+        string_append_escaped (string, katze_item_get_name (item));
+        g_string_append (string, "</H3>\n");
+        g_string_append (string, "\t<DL><P>\n");
+        KATZE_ARRAY_FOREACH_ITEM_L (_item, array, list)
+        {
+            g_string_append (string, "\t");
+            string_append_netscape_item (string, _item);
+        }
+        g_string_append (string, "\t</DL><P>\n");
+
+        g_list_free (list);
+    }
+    else if (katze_item_get_uri (item))
+    {
+        g_string_append (string, "\t<DT><A HREF=\"");
+        string_append_escaped (string, katze_item_get_uri (item));
+        g_string_append (string, "\" ADD_DATE=\"\" LAST_VISIT=\"\" LAST_MODIFIED=\"\">");
+        string_append_escaped (string, katze_item_get_name (item));
+        g_string_append (string, "</A>\n");
+
+        if (item->text && g_strcmp0 (item->text, ""))
+        {
+            g_string_append (string, "\t<DD>");
+            string_append_escaped (string, katze_item_get_text (item));
+            g_string_append (string, "\n");
+        }
+    }
+}
+
 static gchar*
 katze_item_metadata_to_xbel (KatzeItem* item)
 {
@@ -788,15 +869,48 @@ katze_array_to_xbel (KatzeArray* array,
     return g_string_free (markup, FALSE);
 }
 
+static gchar*
+katze_array_to_netscape_html (KatzeArray* array,
+                              GError**    error)
+{
+    KatzeItem* item;
+    GList* list;
+
+    /* The header, including the text, is the same as used in other browsers,
+       see http://msdn.microsoft.com/en-us/library/aa753582(v=vs.85).aspx */
+    GString* markup = g_string_new (
+        "<!DOCTYPE NETSCAPE-Bookmark-file-1>\n"
+        "<!--This is an automatically generated file.\n"
+        "It will be read and overwritten.\n"
+        "Do Not Edit! -->\n"
+        "<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=UTF-8\">\n"
+        "<Title>Bookmarks</Title>\n"
+        "<H1>Bookmarks</H1>\n"
+        "\n");
+    g_string_append (markup, "<DL><P>\n");
+    KATZE_ARRAY_FOREACH_ITEM_L (item, array, list)
+        string_append_netscape_item (markup, item);
+    g_string_append (markup, "</DL><P>\n");
+
+    g_list_free (list);
+    return g_string_free (markup, FALSE);
+}
+
 static gboolean
-midori_array_to_file_xbel (KatzeArray*  array,
-                           const gchar* filename,
-                           GError**     error)
+midori_array_to_file_format (KatzeArray*  array,
+                             const gchar* filename,
+                             const gchar* format,
+                             GError**     error)
 {
     gchar* data;
     FILE* fp;
 
-    if (!(data = katze_array_to_xbel (array, error)))
+    if (!g_strcmp0 (format, "xbel"))
+        data = katze_array_to_xbel (array, error);
+    if (!g_strcmp0 (format, "netscape"))
+        data = katze_array_to_netscape_html (array, error);
+
+    if (!data)
         return FALSE;
     if (!(fp = fopen (filename, "w")))
     {
@@ -833,8 +947,10 @@ midori_array_to_file (KatzeArray*  array,
     g_return_val_if_fail (filename, FALSE);
     g_return_val_if_fail (!error || !*error, FALSE);
 
-    if (!g_strcmp0 (format, "xbel"))
-        return midori_array_to_file_xbel (array, filename, error);
+    if (!g_strcmp0 (format, "xbel")
+    ||  !g_strcmp0 (format, "netscape"))
+        return midori_array_to_file_format (array, filename, format, error);
+
     g_critical ("Cannot write KatzeArray to unknown format '%s'.", format);
     return FALSE;
 }
