@@ -30,9 +30,6 @@ struct _MidoriPanel
 
     GtkWidget* labelbar;
     GtkWidget* toolbar;
-    GtkToolItem* button_align;
-    GtkToolItem* button_detach;
-    GtkToolItem* button_controls;
     GtkWidget* toolbar_label;
     GtkWidget* frame;
     GtkWidget* toolbook;
@@ -194,6 +191,8 @@ midori_panel_class_init (MidoriPanelClass* class)
      * Whether to show operating controls.
      *
      * Since: 0.1.9
+     *
+     * Deprecated: 0.3.0
      */
     g_object_class_install_property (gobject_class,
                                      PROP_SHOW_CONTROLS,
@@ -210,6 +209,8 @@ midori_panel_class_init (MidoriPanelClass* class)
     * Whether to align the panel on the right.
     *
     * Since: 0.1.3
+    *
+    * Deprecated: 0.3.0
     */
     g_object_class_install_property (gobject_class,
                                      PROP_RIGHT_ALIGNED,
@@ -338,31 +339,11 @@ midori_panel_detach_page (MidoriPanel* panel,
     midori_panel_set_current_page (panel, n > 0 ? n - 1 : 0);
     toolitem = gtk_toolbar_get_nth_item (GTK_TOOLBAR (panel->toolbar),
                                          n > 0 ? n - 1 : 0);
-    if (!gtk_notebook_get_n_pages (GTK_NOTEBOOK (panel->notebook)))
-        gtk_widget_set_sensitive (GTK_WIDGET (panel->button_detach), FALSE);
     g_signal_connect (window, "delete-event",
         G_CALLBACK (midori_panel_detached_window_delete_event_cb), panel);
     gtk_widget_show (window);
 }
 
-static void
-midori_panel_button_detach_clicked_cb (GtkWidget*   toolbutton,
-                                       MidoriPanel* panel)
-{
-    /* FIXME: What happens when the browser is destroyed? */
-    /* FIXME: What about multiple browsers? */
-    /* FIXME: Should we remember if the child was detached? */
-    gint n = midori_panel_get_current_page (panel);
-    midori_panel_detach_page (panel, n);
-}
-
-static void
-midori_panel_button_align_clicked_cb (GtkWidget*   toolitem,
-                                      MidoriPanel* panel)
-{
-    midori_panel_set_right_aligned (panel, !panel->right_aligned);
-}
-
 static void
 midori_panel_destroy_cb (MidoriPanel* panel)
 {
@@ -405,33 +386,6 @@ midori_panel_init (MidoriPanel* panel)
     gtk_container_add (GTK_CONTAINER (toolitem), panel->toolbar_label);
     gtk_container_set_border_width (GTK_CONTAINER (toolitem), 6);
     gtk_toolbar_insert (GTK_TOOLBAR (labelbar), toolitem, -1);
-    toolitem = gtk_tool_button_new_from_stock (GTK_STOCK_FULLSCREEN);
-    gtk_widget_set_sensitive (GTK_WIDGET (toolitem), FALSE);
-    panel->button_detach = toolitem;
-    gtk_tool_button_set_label (GTK_TOOL_BUTTON (toolitem),
-        _("Detach chosen panel from the window"));
-    gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (toolitem),
-        _("Detach chosen panel from the window"));
-    g_signal_connect (toolitem, "clicked",
-        G_CALLBACK (midori_panel_button_detach_clicked_cb), panel);
-    #if HAVE_OSX
-    gtk_toolbar_insert (GTK_TOOLBAR (labelbar), toolitem, 0);
-    #else
-    gtk_toolbar_insert (GTK_TOOLBAR (labelbar), toolitem, -1);
-    #endif
-    toolitem = gtk_tool_button_new_from_stock (GTK_STOCK_GO_FORWARD);
-    gtk_tool_button_set_label (GTK_TOOL_BUTTON (toolitem),
-        _("Align sidepanel to the right"));
-    gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (toolitem),
-        _("Align sidepanel to the right"));
-    g_signal_connect (toolitem, "clicked",
-        G_CALLBACK (midori_panel_button_align_clicked_cb), panel);
-    #if HAVE_OSX
-    gtk_toolbar_insert (GTK_TOOLBAR (labelbar), toolitem, 0);
-    #else
-    gtk_toolbar_insert (GTK_TOOLBAR (labelbar), toolitem, -1);
-    #endif
-    panel->button_align = toolitem;
     toolitem = gtk_tool_button_new_from_stock (GTK_STOCK_CLOSE);
     gtk_tool_button_set_label (GTK_TOOL_BUTTON (toolitem), _("Close panel"));
     gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (toolitem), _("Close panel"));
@@ -450,7 +404,6 @@ midori_panel_init (MidoriPanel* panel)
     gtk_notebook_set_show_border (GTK_NOTEBOOK (panel->toolbook), FALSE);
     gtk_notebook_set_show_tabs (GTK_NOTEBOOK (panel->toolbook), FALSE);
     gtk_box_pack_start (GTK_BOX (vbox), panel->toolbook, FALSE, FALSE, 0);
-    gtk_widget_show (panel->toolbook);
 
     /* Create the notebook */
     panel->notebook = gtk_notebook_new ();
@@ -497,12 +450,6 @@ midori_panel_set_property (GObject*      object,
         break;
     case PROP_SHOW_CONTROLS:
         panel->show_controls = g_value_get_boolean (value);
-        sokoke_widget_set_visible (panel->labelbar, panel->show_controls);
-        sokoke_widget_set_visible (panel->toolbar, panel->show_controls);
-        if (panel->button_controls)
-            gtk_toggle_tool_button_set_active (
-                GTK_TOGGLE_TOOL_BUTTON (panel->button_controls),
-                !panel->show_controls);
         break;
     case PROP_RIGHT_ALIGNED:
         midori_panel_set_right_aligned (panel, g_value_get_boolean (value));
@@ -589,15 +536,7 @@ midori_panel_set_right_aligned (MidoriPanel* panel,
     box = gtk_widget_get_parent (panel->toolbar);
     gtk_box_reorder_child (GTK_BOX (box), panel->toolbar,
         right_aligned ? -1 : 0);
-    gtk_tool_button_set_stock_id (GTK_TOOL_BUTTON (panel->button_align),
-        right_aligned ? GTK_STOCK_GO_BACK : GTK_STOCK_GO_FORWARD);
     panel->right_aligned = right_aligned;
-    gtk_tool_button_set_label (GTK_TOOL_BUTTON (panel->button_align),
-        !panel->right_aligned ? _("Align sidepanel to the right")
-            : _("Align sidepanel to the left"));
-    gtk_tool_item_set_tooltip_text (GTK_TOOL_ITEM (panel->button_align),
-        !panel->right_aligned ? _("Align sidepanel to the right")
-            : _("Align sidepanel to the left"));
     g_object_notify (G_OBJECT (panel), "right-aligned");
 }
 
@@ -644,9 +583,6 @@ midori_panel_construct_tool_item (MidoriPanel*    panel,
     g_signal_connect (viewable, "destroy",
                       G_CALLBACK (midori_panel_widget_destroy_cb), toolitem);
 
-    if (gtk_notebook_get_n_pages (GTK_NOTEBOOK (panel->notebook)))
-        gtk_widget_set_sensitive (GTK_WIDGET (panel->button_detach), TRUE);
-
     return GTK_TOOL_ITEM (toolitem);
 }
 
@@ -677,16 +613,6 @@ midori_panel_action_activate_cb (GtkRadioAction* action,
     }
 }
 
-#if !HAVE_HILDON
-static void
-midori_panel_show_controls_toggled_cb (GtkWidget*   menuitem,
-                                       MidoriPanel* panel)
-{
-    if (panel->show_controls != !gtk_toggle_tool_button_get_active (GTK_TOGGLE_TOOL_BUTTON (menuitem)))
-        g_object_set (panel, "show-controls", !panel->show_controls, NULL);
-}
-#endif
-
 /**
  * midori_panel_append_page:
  * @panel: a #MidoriPanel
@@ -749,17 +675,6 @@ midori_panel_append_page (MidoriPanel*    panel,
     gtk_container_add (GTK_CONTAINER (panel->notebook), scrolled);
 
     toolbar = midori_viewable_get_toolbar (viewable);
-    #if !HAVE_HILDON
-    toolitem = gtk_toggle_tool_button_new_from_stock (GTK_STOCK_PROPERTIES);
-    gtk_tool_item_set_tooltip_text (toolitem, _("Hide operating controls"));
-    gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON (toolitem),
-        !panel->show_controls);
-    g_signal_connect (toolitem, "clicked",
-        G_CALLBACK (midori_panel_show_controls_toggled_cb), panel);
-    gtk_toolbar_insert (GTK_TOOLBAR (toolbar), toolitem, 0);
-    gtk_widget_show (GTK_WIDGET (toolitem));
-    panel->button_controls = toolitem;
-    #endif
     gtk_widget_show (toolbar);
     gtk_container_add (GTK_CONTAINER (panel->toolbook), toolbar);
     g_signal_connect (viewable, "destroy",
@@ -954,8 +869,7 @@ midori_panel_set_current_page (MidoriPanel* panel,
         gtk_notebook_set_current_page (GTK_NOTEBOOK (panel->toolbook), n);
         toolbar = gtk_notebook_get_nth_page (GTK_NOTEBOOK (panel->toolbook), n);
         items = gtk_container_get_children (GTK_CONTAINER (toolbar));
-        sokoke_widget_set_visible (panel->toolbook,
-            g_list_nth_data (items, 1) != NULL);
+        sokoke_widget_set_visible (panel->toolbook, items != NULL);
         g_list_free (items);
         gtk_notebook_set_current_page (GTK_NOTEBOOK (panel->notebook), n);
         label = midori_viewable_get_label (MIDORI_VIEWABLE (viewable));
