@@ -17,7 +17,6 @@
 #include "midori-browser.h"
 #include "midori-extension.h"
 #include "midori-locationaction.h"
-#include "midori-locationentry.h"
 #include "midori-panel.h"
 #include "midori-preferences.h"
 #include "midori-searchaction.h"
@@ -27,7 +26,7 @@
 #include "midori-websettings.h"
 
 /* For convenience, include localization header */
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 
 #define MIDORI_CHECK_VERSION(major, minor, micro) \
   (MIDORI_MAJOR_VERSION > (major) || \
