@@ -154,25 +154,6 @@ midori_history_format_date (KatzeItem *item)
     return sdate;
 }
 
-static void
-midori_history_clear_db (MidoriHistory* history)
-{
-    gchar* sqlcmd;
-    sqlite3* db;
-    char* errmsg = NULL;
-
-    db = g_object_get_data (G_OBJECT (history->array), "db");
-    sqlcmd = sqlite3_mprintf ("DELETE FROM history");
-
-    if (sqlite3_exec (db, sqlcmd, NULL, NULL, &errmsg) != SQLITE_OK)
-    {
-        g_printerr (_("Failed to remove history item: %s\n"), errmsg);
-        sqlite3_free (errmsg);
-    }
-
-    sqlite3_free (sqlcmd);
-}
-
 static void
 midori_history_remove_item_from_db (MidoriHistory* history,
                                     KatzeItem*     item)
@@ -231,11 +212,11 @@ midori_history_read_from_db (MidoriHistory* history,
         gchar* filterstr;
 
         sqlcmd = "SELECT * FROM ("
-                 "    SELECT uri, title, day FROM history"
+                 "    SELECT uri, title, day, date FROM history"
                  "    WHERE uri LIKE ?1 OR title LIKE ?1 GROUP BY uri "
                  "UNION ALL "
                  "    SELECT replace (uri, '%s', keywords) AS uri, "
-                 "    keywords AS title, day FROM search "
+                 "    keywords AS title, day, 0 AS date FROM search "
                  "    WHERE uri LIKE ?1 OR keywords LIKE ?1 GROUP BY uri "
                  ") ORDER BY day ASC";
         result = sqlite3_prepare_v2 (db, sqlcmd, -1, &statement, NULL);
@@ -327,7 +308,7 @@ midori_history_clear_clicked_cb (GtkWidget*     toolitem,
     if (result != GTK_RESPONSE_YES)
         return;
 
-    midori_history_clear_db (history);
+    katze_array_clear (history->array);
 }
 
 static void
@@ -453,6 +434,14 @@ midori_history_add_item_cb (KatzeArray*    array,
                                        0, 0, NULL, -1);
 }
 
+static void
+midori_history_clear_cb (KatzeArray*    array,
+                         MidoriHistory* history)
+{
+    GtkTreeView* treeview = GTK_TREE_VIEW (history->treeview);
+    GtkTreeModel* model = gtk_tree_view_get_model (treeview);
+    gtk_tree_store_clear (GTK_TREE_STORE (model));
+}
 static void
 midori_history_set_app (MidoriHistory* history,
                         MidoriApp*     app)
@@ -463,12 +452,14 @@ midori_history_set_app (MidoriHistory* history,
     {
         g_signal_handlers_disconnect_by_func (history->array,
             midori_history_add_item_cb, history);
+        g_signal_handlers_disconnect_by_func (history->array,
+            midori_history_clear_cb, history);
         katze_object_assign (history->array, NULL);
         model = gtk_tree_view_get_model (GTK_TREE_VIEW (history->treeview));
         gtk_tree_store_clear (GTK_TREE_STORE (model));
     }
 
-    katze_assign (history->app, app);
+    katze_object_assign (history->app, app);
     if (!app)
         return;
     g_object_ref (app);
@@ -476,6 +467,8 @@ midori_history_set_app (MidoriHistory* history,
     history->array = katze_object_get_object (app, "history");
     g_signal_connect (history->array, "add-item",
                       G_CALLBACK (midori_history_add_item_cb), history);
+    g_signal_connect (history->array, "clear",
+                      G_CALLBACK (midori_history_clear_cb), history);
     model = gtk_tree_view_get_model (GTK_TREE_VIEW (history->treeview));
     if (history->array)
         midori_history_read_from_db_to_model (history, GTK_TREE_STORE (model), NULL, 0, NULL);
@@ -948,6 +941,7 @@ midori_history_init (MidoriHistory* history)
     gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview), FALSE);
     gtk_tree_view_set_tooltip_column (GTK_TREE_VIEW (treeview), 1);
     column = gtk_tree_view_column_new ();
+    gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
     renderer_pixbuf = gtk_cell_renderer_pixbuf_new ();
     gtk_tree_view_column_pack_start (column, renderer_pixbuf, FALSE);
     gtk_tree_view_column_set_cell_data_func (column, renderer_pixbuf,
@@ -992,6 +986,8 @@ midori_history_finalize (GObject* object)
 
     g_signal_handlers_disconnect_by_func (history->array,
         midori_history_add_item_cb, history);
+    g_signal_handlers_disconnect_by_func (history->array,
+        midori_history_clear_cb, history);
     g_object_unref (history->array);
     katze_assign (history->filter, NULL);
 }
